#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sysexits.h>
#include <roboctl.h>
#include "vexctl_cmd.h"
#include "protos.h"

extern int     Debug;

int     main(int argc,char *argv[])

{
    rct_cmd_t   cmd = RCT_CMD_STATUS;
    arg_t       arg_data = ARG_INIT;
    unsigned int    flags = 0;

    switch(argc)
    {
	case    1:
	    vexctl_usage(argv[0]);
	    break;
	default:
	    break;
    }

    if ( parse_args(argc,argv,&cmd,&arg_data,&flags) == RCT_OK )
    {
	return vexctl(cmd,&arg_data,flags);
    }
    else
	vexctl_usage(argv[0]);
    return 0;
}


int     vexctl(rct_cmd_t cmd,arg_t *arg_data,unsigned int flags)

{
    rct_status_t    status;
    rct_pic_t       pic;
    
    printf("Opening %s...\n",arg_data->device);
    fflush(stdout);
    if ( pic_open_controller(&pic,arg_data->device) != RCT_OK )
    {
	fprintf(stderr,"vexctl(): Could not open %s.\n",arg_data->device);
	return RCT_OPEN_FAILED;
    }
    switch(cmd)
    {
	case    RCT_CMD_STATUS:
	    status = pic_print_bootloader_version(&pic);
	    break;
	case    RCT_CMD_UPLOAD:
	    status = vex_upload_program(&pic,arg_data->filename);
	    break;
	default:
	    fprintf(stderr,"Internal error: Invalid command: %d.\n",cmd);
	    fputs("This is a program bug.  Please report it to the author(s).\n",stderr);
	    exit(EX_SOFTWARE);
    }
    pic_return_to_user_code(&pic);
    pic_close_controller(&pic);
    if ( (status == RCT_OK) && flags&VEX_LAUNCH_TERM )
	return launch_terminal();
    else
	return status;
}


void    vexctl_usage(char *progname)

{
    fputs("Usage:\n",stderr);
    fprintf(stderr,"\t%s [flags] status\n",progname);
    fprintf(stderr,"\t%s [flags] upload <filename>\n",progname);
    fputs("\nFlags:\n",stderr);
    fputs("\t--debug     Enable debugging output.\n",stderr);
    fputs("\t--dev dev   Use serial port dev instead of the default.\n",stderr);
    fputs("\t--term      Launch terminal after successful command.\n",stderr);
    exit(EX_USAGE);
}


int     parse_args(int argc,char *argv[],rct_cmd_t *cmd,
		    arg_t *arg_data,unsigned int *flags)

{
    int     arg;

    *cmd = RCT_CMD_NONE;
    
    if ( parse_global_flags(argc,argv,cmd,arg_data,flags,&arg) != RCT_OK )
	return RCT_USAGE;

    if ( strcmp(argv[arg],"status") == 0 )
    {
	*cmd = RCT_CMD_STATUS;
	if ( arg != argc-1 )
	    vexctl_usage(argv[0]);
    }
    else if ( strcmp(argv[arg],"upload") == 0 )
    {
	*cmd = RCT_CMD_UPLOAD;
	/* The next argument should be the last */
	if ( arg == argc - 2 )
	    arg_data->filename = argv[++arg];
	else
	    vexctl_usage(argv[0]);
    }
    
    if ( *cmd == RCT_CMD_NONE )
    {
	vexctl_usage(argv[0]);
	return 0;   /* vexctl_usage() exits, but the compiler
		       will complain about reaching the end of a
		       non-void function without this return. */
    }
    else
	return RCT_OK;
}


int     parse_global_flags(int argc,char *argv[],rct_cmd_t *cmd,
		    arg_t *arg_data,unsigned int *flags,int *argp)

{
    int arg = *argp;
    extern int  Debug;
    
    for (arg=1; (arg < argc) && (argv[arg][0] == '-'); ++arg)
    {
	if ( strcmp(argv[arg],"--debug") == 0 )
	{
	    Debug = 1;
	}
	else if ( strcmp(argv[arg],"--dev") == 0 )
	{
	    arg_data->device = argv[++arg];
	}
	else if ( strcmp(argv[arg],"--term") == 0 )
	{
	    *flags |= VEX_LAUNCH_TERM;
	}
	else
	{
	    fprintf(stderr,"Invalid flag: %s\n",argv[arg]);
	    vexctl_usage(argv[0]);
	}
    }
    
    if ( arg == argc )
	vexctl_usage(argv[0]);
    
    *argp = arg;
    return RCT_OK;
}


int     launch_terminal(void)

{
    return system("cutecom");
}

