#ifndef _TERMIOS_H_
#include <termios.h>
#endif

#define PIC_CMD_MAX         255
#define PIC_RESPONSE_MAX    255
#define PIC_HEX_LINE_MAX    1024

/* PIC special characters */
#define CHAR_SI     0x0F    /* Start transmission */
#define CHAR_EOT    0x04    /* End transmission */
#define CHAR_ESC    0x05    /* Escape next character (must precede CHAR_SI,
				CHAR_EOT and CHAR_ESC in payload */

#define PIC_BAUD_RATE   B115200

#ifndef MIN
#define MIN(x,y)    ((x) < (y) ? (x) : (y))
#endif

#ifndef MAX
#define MAX(x,y)    ((x) > (y) ? (x) : (y))
#endif

typedef enum
{
    PIC_GET_BOOTLOADER_VERSION= 0x00,
    PIC_READ_PROGRAM_MEM=       0x01,
    PIC_WRITE_PROGRAM_MEM=      0x02,
    PIC_RETURN_TO_USER_CODE=    0x08,
    PIC_ERASE_PROGRAM_MEM=      0x09
}   pic_cmd_t;

typedef struct
{
    unsigned long   address;
    int             len;
    char            code[PIC_HEX_LINE_MAX+1];
}   pic_line_t;

typedef struct
{
    int             fd;
    int             bootloader_major;
    int             bootloader_minor;
    char            *device;
    char            response[PIC_RESPONSE_MAX+1];
    struct termios  port_settings;
    struct termios  raw_port_settings;
}   rct_pic_t;

#define PIC_IS_OPEN(p)  ((p)->fd != -1)

